#!/usr/bin/env python
# -*- coding: utf-8 -*-

from builtins import str
import time

from db_paribhasha.dynamo import dynamo_constants as cnsts
from db_paribhasha.dynamo import dynamo_helpers as db_helpers
from db_paribhasha.dynamo.import_workflow import ImportWorkflow
from db_paribhasha.error_handler import OperationNotPerformedError


def create_table():
    """
    Creates table if table doesn't exist
    """

    if ImportWorkflow.exists():
        raise OperationNotPerformedError("Table already exists")
    ImportWorkflow.create_table(
        read_capacity_units=cnsts.DYNAMO_RW_CAP_20,
        write_capacity_units=cnsts.DYNAMO_RW_CAP_20)


def get_import_workflow_detail(import_workflow_id):
    """
    Returns the detail of a given workflow id.

    Parameters
    ----------
    import_workflow_id: String

    Returns
    -------
    import_workflow: Pynamo object
        Import workflow for the given unique ID.
    """
    org_id, user_id, created_timestamp = import_workflow_id.split('-')
    import_workflow = ImportWorkflow.get(
        int(org_id), user_id + '-' + created_timestamp)
    return import_workflow


def create_import_workflow(org_id,
                           email_id,
                           user_data,
                           import_data,
                           workflow_adapter_id,
                           project_id,
                           metadata=None):
    """
    Creates an import workflow for a given user with given user_data

    Parameters
    ----------
    org_id: Integer
        Organization ID
    email_id: String
        Email ID of the creating user
    user_data: JSON
        JSON containing the User data
    import_data: JSON
        JSON containing the import data

    Returns
    -------
    import_workflow_id: String
        Created import workflow ID
    """
    added_timestamp = int(round(time.time() * 1000))
    last_modified_timestamp = added_timestamp
    calculated_id = str(email_id) + '-' + str(added_timestamp)
    import_workflow =\
        ImportWorkflow(org_id=int(org_id), created_ts=added_timestamp,
                       calculated_id=calculated_id,
                       modified_ts=last_modified_timestamp,
                       user_email=email_id,
                       user_data=user_data,
                       import_data=import_data,
                       import_adapter_id=workflow_adapter_id,
                       project_id=int(project_id))
    db_helpers.save_dynamo_object(import_workflow)
    return '{}-{}'.format(str(org_id), calculated_id)
