#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time

from db_paribhasha import credentials
from db_paribhasha.error_handler import OperationNotPerformedError
from db_paribhasha.dynamo.dynamo_helpers import save_dynamo_object
from db_paribhasha.dynamo.studio_visualisation.studio_visualisation_model import StudioVisualisation


def create_table():
    """
    Creates table if table doesn't exist
    """

    if StudioVisualisation.exists():
        raise OperationNotPerformedError("Table already exists")
    StudioVisualisation.create_table(
        read_capacity_units=credentials.DYNAMO_GENERIC_RW_CAP,
        write_capacity_units=credentials.DYNAMO_GENERIC_RW_CAP)


def create_chart_request(studio_viz_id, status, chart_info, message):
    """
    Creates a chart request

    Parameters
    ----------
    studio_viz_id : String
        Unique studio visualisation request id
    status : String
        Status of charts for this job
    chart_info : Dict
        Chart relative urls
    """

    chart_request = StudioVisualisation(chart_info=chart_info,
                                        status=status,
                                        studio_viz_id=studio_viz_id,
                                        message=message)

    save_dynamo_object(chart_request)


def get_chart_data(studio_viz_id):
    """
    Returns the pynamo object containing chart info

    Parameters
    ----------
    studio_viz_id : String
        Unique studio visualisation request id

    Returns
    -------
    chart_object: pynamo object
        contains chart relative urls and status
    """
    chart_object = StudioVisualisation.get(str(studio_viz_id))
    return chart_object


def as_dict(chart_query):
    """
    Return dictionary object of dynamodb
    attributes

    Parameters
    ----------
    chart_query : pynamo object
        pynamo object of studio visualisation dynamodb

    """
    return {
        'status': chart_query.status,
        'chart_info': chart_query.chart_info,
        'studio-viz-id': chart_query.studio_viz_id,
        'message': chart_query.message
    }
