from db_paribhasha.error_handler import DatabaseEntryError
from db_paribhasha import db_constants as db_cnts
from db_paribhasha.workflow.algorithms import algorithm_table
from db_paribhasha.base_models.sql_base_model import session

def get_value(algo_name, column_name):
    """
        This function checks returns the value of the given column of 
        a particular algorithm with algo_id

    Parameters
    -------------
    algo_name : str
        Algorithm whoes column value is to be retreived
    column_name : str
        column whoes values is to be retreived

    Returns
    -------------
    colum_value : str | int
        value of the specified column
    """
    if column_name not in list(algorithm_table.Algorithms.__table__.columns.keys()):
        raise DatabaseEntryError(db_cnts.INVALID_COLUMN)

    if has_algorithm(algo_name):
        algo_row = session.query(algorithm_table.Algorithms).filter_by(name=algo_name).first()
        return getattr(algo_row, column_name)
    else:
        raise DatabaseEntryError(db_cnts.INVALID_ID.format('Algorithm'))

def has_algorithm(algo_name):
    """
        This function checks if a given iteration id is present in the table
        or not.

    Parameters
    -------------
    algo_name: list
        algorithm name whose existence has to be checked
    
    Returns
    -------------
    boolean:
        algorithm with id exists or not
    """
    algo_row = session.query(algorithm_table.Algorithms).filter_by(name=algo_name).first()
    if algo_row is not None:
        return True
    else:
        return False

def get_algorithms(algo_names):   
    """
        This function checks if a given iteration id is present in the table
        or not.

    Parameters
    -------------
    algo_names : list
        list of algorithm names whose data has to be fetched
    
    Returns
    -------------
    boolean:
        algorithm with id exists or not
    """
    algorithms = session.query(algorithm_table.Algorithms).filter(session.query(algorithm_table.Algorithms).contains(algo_names)).all()
    return [algo.as_dict() for algo in algorithms]

def get_value_with_id(algo_id, column_name):

    """
        This function checks returns the value of the given column of 
        a particular algorithm with algo_id

    Parameters
    -------------
    algo_id : int
        Algorithm whoes column value is to be retreived
    column_name : str
        column whoes values is to be retreived

    Returns
    -------------
    colum_value : str | int
        value of the specified column
    """
    if column_name not in list(algorithm_table.Algorithms.__table__.columns.keys()):
        raise DatabaseEntryError(db_cnts.INVALID_COLUMN)

    if has_algorithm_with_id(algo_id):
        algo_row = session.query(algorithm_table.Algorithms).filter_by(id=algo_id).first()
        return getattr(algo_row, column_name)
    else:
        raise DatabaseEntryError(db_cnts.INVALID_ID.format('Algorithm'))

def has_algorithm_with_id(algo_id):
    """
        This function checks if a given algorithm id is present in the table
        or not.

    Parameters
    -------------
    algo_id : str
        algorithm id whose existence that has to be checked
    
    Returns
    -------------
    boolean:
        algorithm with id exists or not
    """
    algo_row = session.query(algorithm_table.Algorithms).filter_by(id=algo_id).first()
    if algo_row is not None:
        return True
    else:
        return False


def id_with_name(names):
    '''
    It return the name list of ids of algorithm with the
    given names

    RETURNS
    --------
    algo_ids: list
        list of algorithm ids
    '''
    algo_ids = session.query(algorithm_table.Algorithms.id).filter(algorithm_table.Algorithms.name.in_(names)).all()
    return algo_ids


def algo_exists(name):
        '''
        It returns a boolean value whether an algorithm with
        the given name exists or not

        RETURNS
        --------
        bool:
            boolean telling whether algorithm with the given
            name exists or not.
        '''
        count = session.query(algorithm_table.Algorithms.id).filter(algorithm_table.Algorithms.name==name).count()
        return bool(count)
