from builtins import str
from sqlalchemy import Boolean, Column, DateTime, Integer, String
from sqlalchemy.orm import relationship

from db_paribhasha import db_constants
from db_paribhasha.base_models.sql_base_model import Base, session

class User(Base):
    """
        This class corresponds to the user table in the database.
        It is used to store the user records in the database
    """
    __tablename__ = db_constants.USER_TABLE_NAME
    id = Column(Integer, primary_key=True, autoincrement=True, nullable=False)
    email = Column(String(255), unique=True)
    first_name = Column(String(255))
    active = Column(Boolean)
    last_name = Column(String(255))
    confirmed_at = Column(DateTime)
    workflow_request = relationship('WorkflowRequest')

    @classmethod
    def get_user(self, email_id):
        """
        This is the Class method of the User model.
        It returns the user.

        RETURNS
        --------
        Object/None:
            User object or None
        """
        user = session.query(User).filter(self.email == email_id).first()
        return user

    def __init__(self,
                 email,
                 first_name,
                 last_name,
                 active,
                 confirmed_at=None,):
        """ CLASS CONSTRUCTOR """

        self.email = email.lower()
        self.first_name = first_name
        self.last_name = last_name
        self.active = active
        self.confirmed_at = confirmed_at

    def as_dict(self):
        return_object = {
            "id": self.id,
            "email": self.email,
            "first_name": self.first_name,
            "last_name": self.last_name,
            "active": self.active,
            "confirmed_at": str(self.confirmed_at)
        }
        return return_object
